using System;
using System.Data;
using Shop.Common;
using Shop.DAL;

namespace Shop.BLL
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ICustomer
	{
        DataSet GetByID(string user, string pass, int clientID);

        DataSet FindByName(string user, string pass, string namePattern);

        int Add(string user, string pass, string name, string address, string phone, 
				 string fax, string eMail, out ShopStatus status);

        ShopStatus Update(string user, string pass, int customerID, string name, string address, 
							  string phone, string fax, string eMail);
	}

    public class Customer : BaseTableModule, ICustomer
	{
        public Customer()
        {
        }

        public Customer(TransactionalContext tx)
            : base(tx)
        {
        }

		#region ICustomer Members

		public DataSet GetByID(string user, string pass, int clientID)
		{
            try
            {
                User u = new User(CurrentContext);
                ShopStatus status = u.Validate(user, pass);
                if (status != ShopStatus.Ok)
                    return null;

                return GetByID(clientID);
            }
            catch (ShopException ex)
            {
                System.Console.WriteLine("Erro DAL: " + ex.Message);
                return null;
            }
		}

        public DataSet FindByName(string user, string pass, string namePattern)
		{
            User u = new User(CurrentContext);
            if (u.Validate(user, pass) != ShopStatus.Ok)
                return null;

            return new CustomerGateway(CurrentContext).FindByName(namePattern);
		}

        private bool IsValid(string name)
        {
            return (name != null && name.Trim().Length != 0);
        }

		public int Add(string user, string pass, 
			            string name, string address, string phone, string fax, 
			            string eMail, out ShopStatus status)
		{
            User u = new User(CurrentContext);
            status = u.Validate(user, pass);
            if (status != ShopStatus.Ok)
                return -1; 
            
            // validar dados de entrada
			if (!IsValid(name))
			{
				status = ShopStatus.InvalidArgument;
				return -1;
			}

            return new CustomerGateway(CurrentContext).Insert(name, address, phone, fax, eMail);
		}

		public ShopStatus Update(string user, string pass, 
			                         int customerID, string name, string address, 
									 string phone, string fax, string eMail)
		{
            User u = new User(CurrentContext);
            ShopStatus status = u.Validate(user, pass);
            if (status != ShopStatus.Ok)
                return status;

            // validar dados de entrada
            if (!IsValid(name))
                return ShopStatus.InvalidArgument;

            return new CustomerGateway(CurrentContext).Update(customerID, name, address, phone, fax, eMail);
		}

		#endregion

        public ShopStatus ValidateCustomerLogin(string user, string pass)
        {
            DAL.CustomerGateway gtw = new Shop.DAL.CustomerGateway(CurrentContext);
            DataSet ds = gtw.GetByUsernameAndPassword(user, pass);
            if (ds.Tables[0].Rows.Count != 1)
                return ShopStatus.InvalidLogin;
            else
                return ShopStatus.Ok;
        }

        internal DataSet GetByID(int CustomerID)
        {
            return new CustomerGateway(CurrentContext).GetByID(CustomerID);
        }
    }
}
