using System;
using System.Data;
using System.Data.OleDb;
using Shop.Common;
using Shop.DAL;

namespace Shop.BLL
{
	public interface IUser
	{
		ShopStatus Validate(string user, string pass);
		ShopStatus Validate(string user, string pass, out int userID);
		DataSet ValidateAndGetUser(string user, string pass, out ShopStatus status);
	}

	public class User : BaseTableModule, IUser
	{
        public User()
        {
        }

        public User(TransactionalContext tx)
            : base(tx)
        {
        }

#region IUser Members

		public ShopStatus Validate(string user, string pass)
		{
			int userID;
			return Validate(user, pass, out userID);
		}

		public ShopStatus Validate(string user, string pass, out int userID)
		{
            userID = -1;

            // validaes
            if (!IsValid(user, pass))
                return ShopStatus.InvalidArgument;

            UserGateway u = new UserGateway(CurrentContext);
            DataSet ds = u.GetByUsernameAndPassword(user, pass);
            if (ds.Tables[0].Rows.Count > 0)
            {
                userID = (int)ds.Tables[0].Rows[0]["UserID"];
                return ShopStatus.Ok;
            }
            else
            {   
                return ShopStatus.InvalidLogin;
            }
		}

		public DataSet ValidateAndGetUser(string user, string pass, out ShopStatus status)
		{
            status = ShopStatus.InvalidArgument;

            // validaes
            if (!IsValid(user, pass))
                return null;

            UserGateway u = new UserGateway(CurrentContext);
            DataSet ds = u.GetByUsernameAndPassword(user, pass);
            if (ds.Tables[0].Rows.Count > 0)
            {
                status = ShopStatus.Ok;
                return ds;
            }
            else
            {
                status = ShopStatus.InvalidLogin;
                return null;
            }
        }

		#endregion

        private bool IsValid(string user, string pass)
        {
            if (user.Length == 0 || pass.Length == 0)
                return false;
            else
                return true;
        }
	}
}
