using System;
using System.Data;
using System.Data.OleDb;
using Shop.Common;
namespace Shop.DAL
{
	public class CustomerGateway : BaseGateway
	{
        public CustomerGateway() { }
        public CustomerGateway(TransactionalContext tx) : base(tx) { }

		public DataSet GetByID(long clientID)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				conn = GetConnection();

				ds = UtilDB.GetByID(conn, GetTransaction(), "Customers", "CustomerID", clientID);
			}
			catch(OleDbException ex)
			{
				throw new ShopException("customer.GetByID", ex);
			}
			finally
			{
                CloseConnection(conn);
			}

            return ds; // ds.Tables[0].Rows[0];
		}

        public DataSet GetByUsernameAndPassword(string user, string pass)
        {
            OleDbConnection conn = null;
            try
            {
                conn = GetConnection();
                OleDbDataAdapter da = new OleDbDataAdapter("Select * from Customers Where User=? and Pass=?", conn);
                da.SelectCommand.Transaction = GetTransaction();
                da.SelectCommand.Parameters.Add("@u", user);
                da.SelectCommand.Parameters.Add("@p", pass);
                DataSet ds = new DataSet();
                da.Fill(ds, "Customer");
                return ds;
            }
            catch (OleDbException ex)
            {
                throw new ShopException("customer.GetByID", ex);
            }
            finally
            {
                CloseConnection(conn);
            }
        }

		public DataSet FindByName(string namePattern)
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				conn = GetConnection();
			
				ds = UtilDB.FindByStringField(conn, "Customers", "Name", namePattern);
			}
			catch(OleDbException ex)
			{
                throw new ShopException("Customer.FindByName", ex);
            }
			finally
			{
                CloseConnection(conn);
			}

            return ds; // ds.Tables[0];
		}

		public int Insert(string name, string address, string phone, string fax, string eMail)
		{
			int customerID = -1;
			OleDbConnection conn = null;

			try
			{	
				conn = GetConnection();

				// criar comando SQL a executar
				string sqlCmd = "INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, EMail) Values (?, ?, ?, ?, ?)";
				OleDbCommand cmd = new OleDbCommand(sqlCmd, conn, GetTransaction());
				cmd.Parameters.AddWithValue("name", name);
				if (address.Length == 0)
					cmd.Parameters.AddWithValue("address", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("address", address);
				if (phone.Length == 0)
					cmd.Parameters.AddWithValue("phone", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("phone", phone);
				if (fax.Length == 0)
					cmd.Parameters.AddWithValue("fax", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("fax", fax);
				if (eMail.Length == 0)
					cmd.Parameters.AddWithValue("email", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("email", eMail);
		
				int insRows = cmd.ExecuteNonQuery();

                OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", conn);
				customerID = (int)idCmd.ExecuteScalar();
			}
			catch(OleDbException ex)
			{
                throw new ShopException("Customer.Create", ex);
            }
			finally
			{
                CloseConnection(conn);
			}

			return customerID;
		}

		public ShopStatus Update(long customerID, string name, string address, 
									 string phone, string fax, string eMail)
		{
			OleDbConnection conn = null;
			ShopStatus status;

			try
			{
				conn = GetConnection();

				// criar comando SQL a executar
				string sSqlCmd = "Update Customers SET Name = ?, Address = ?, PhoneNb = ?, FaxNb = ?, EMail = ? WHERE CustomerID = ?";
				OleDbCommand cmd = new OleDbCommand(sSqlCmd, conn, GetTransaction());
				cmd.Parameters.AddWithValue("name", name);
				if (address.Length == 0)
					cmd.Parameters.AddWithValue("address", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("address", address);
				if (phone.Length == 0)
					cmd.Parameters.AddWithValue("phone", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("phone", phone);
				if (fax.Length == 0)
					cmd.Parameters.AddWithValue("fax", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("fax", fax);
				if (eMail.Length == 0)
					cmd.Parameters.AddWithValue("email", DBNull.Value);
				else
					cmd.Parameters.AddWithValue("email", eMail);
				cmd.Parameters.AddWithValue("custid", customerID);

				int updRows = cmd.ExecuteNonQuery();

				if (updRows == 0)
					status = ShopStatus.NotOk;
				else
					status = ShopStatus.Ok;
			}
			catch(OleDbException ex)
			{
                throw new ShopException("Customer.Update", ex);
			}
			finally
			{
                CloseConnection(conn);
			}

			return status;
		}
	}
}
