using System;
using System.Data;
using System.Data.OleDb;
using Shop.Common;
using Isep.Util.Data;

namespace Shop.DAL
{
    public class ProductGateway : BaseGateway
	{
        public ProductGateway() { }
        public ProductGateway(TransactionalContext tx) : base(tx) { }

		public DataSet GetByID(int ProductID)
		{
			OleDbConnection conn = null;

			try
			{
				conn = GetConnection();
				DataSet rs = UtilDB.GetByID(conn, GetTransaction(), "Products", "ProductID", ProductID);
                if (rs.Tables[0].Rows.Count != 1)
                    return null;
                else
                    return rs; // rs.Tables[0].Rows[0];
			}
			catch(OleDbException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <GetById> da classe 'Product': " + ex.Message);
                throw new ShopException("Product.GetById", ex);
			}
			finally
			{
                CloseConnection(conn);
			}
		}

		public DataSet FindByName(string NamePattern)
		{
			OleDbConnection conn = null;

			try
			{
				conn = GetConnection();
				
				DataSet rs = UtilDB.FindByStringField(conn, "Products", "Description", NamePattern);
                return rs; // rs.Tables[0];
            }
			catch(OleDbException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <FindByName> da classe 'Product': " + ex.Message);
                throw new ShopException("Product.FindByName", ex);
            }
			finally
			{
				CloseConnection(conn);
			}
		}

        // deveria usar optimistic lock (necessitava da DataRow para ter dados originais)
        // este update  destrutivo, i.e., altera todos os valores existentes dos campos no registo
		public void Update(int ProductID, string description, float? unitPrice, int? quantInStock)
		{
			OleDbConnection conn = null;

			try
			{
                OleDbCommand cmd = new OleDbCommand();
                string sSqlCmd = "Update Products SET ";
                if (description != null)
                {
                    sSqlCmd += "description=?, ";
                    cmd.Parameters.Add("desc", description);
                }
                if (unitPrice.HasValue)
                {
                    sSqlCmd += "unitPrice=?, ";
                    cmd.Parameters.Add("unitprice", unitPrice.Value);
                }
                if (quantInStock.HasValue)
                {
                    sSqlCmd += "StockQtd = ?, ";
                    cmd.Parameters.Add("qt", quantInStock);
                }
                sSqlCmd=sSqlCmd.Remove(sSqlCmd.Length - 2);
                sSqlCmd += " WHERE ProductID = ?";
                cmd.Parameters.Add("productID", ProductID);
                cmd.CommandText = sSqlCmd;
                conn = GetConnection();
                cmd.Connection = conn;
                cmd.Transaction = GetTransaction();

                if (cmd.ExecuteNonQuery() == 0)
                    throw new ShopException(ShopError.UnexistentId, "Product.Update");
			}
			catch(OleDbException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <Update> class 'Product': " + ex.Message);
                throw new ShopException("Product.Update", ex);
			}
			finally
			{
                CloseConnection(conn);
			}
		}

        // este update  no destrutivo
        // caso especifico do stock para previnir erros de acessos concorrentes
        public int DeltaUpdateStock(int productID, int deltaQuant)
        {
            OleDbConnection conn = null;

            try
            {
                conn = GetConnection();

                OleDbCommand cmd = new OleDbCommand();
                string sSqlCmd = "Update Products SET StockQtd=StockQtd+? WHERE ProductId=?";
                cmd.Parameters.AddWithValue("qt", deltaQuant);
                cmd.Parameters.AddWithValue("productID", productID);
                cmd.CommandText = sSqlCmd;
                cmd.Connection = conn;
                cmd.Transaction = GetTransaction();

                return cmd.ExecuteNonQuery();
            }
            catch (OleDbException ex)
            {
                System.Console.WriteLine("EXCEPO no mtodo <DeltaUpdateStock> class 'Product': " + ex.Message);
                throw new ShopException(ex);
            }
            finally
            {
                CloseConnection(conn);
            }
        }

		public DataSet GetAll()
		{
			OleDbConnection conn = null;
			DataSet ds = null;

			try
			{
				conn = GetConnection();
                OleDbTransaction tx = GetTransaction();
                if (tx == null)
				    ds = OleDbHelper.ExecuteDataSet(conn, "SELECT * FROM vwProductCatalog", "Product");
                else
                    ds = OleDbHelper.ExecuteDataSet(tx, "SELECT * FROM vwProductCatalog", "Product");
                return ds; // ds.Tables[0];
			}
			catch(OleDbException ex)
			{
				System.Console.WriteLine("EXCEPO no mtodo <GetCatalog> da classe 'Product': " + ex.Message);
                throw new ShopException("Product.GetAll", ex);
            }
			finally
			{
				CloseConnection(conn);
			}
		}

        public void FillByID(int[] listaProd, DataSet pDS)
        {
            OleDbConnection conn = null;

            try
            {
                conn = GetConnection();

                System.Text.StringBuilder sb = new System.Text.StringBuilder("Select * From Products Where ProductID IN (");
                foreach (int k in listaProd)
                {
                    sb.Append(k.ToString());
                    sb.Append(",");
                }
                sb.Length = sb.Length - 1;
                sb.Append(")");
                string sSqlCmd = sb.ToString();

                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Transaction = GetTransaction();
                oAdapter.Fill(pDS, "Products");
            }
            catch (OleDbException ex)
            {
                System.Console.WriteLine("EXCEPO no mtodo 'GetByID': " + ex.Message);
                throw new ShopException(ex);
            }
            finally
            {
                CloseConnection(conn);
            }
        }
    }
}
