using System;
using System.Data;
using System.Data.OleDb;
using Shop.Common;

namespace Shop.DAL
{
    public class SaleGateway : BaseGateway
    {
        public SaleGateway() { }
        public SaleGateway(TransactionalContext tx) : base(tx) { }

        public int Insert(int CustomerID, DateTime Date)
        {
            OleDbConnection conn = null;

            try
            {
                // criar objecto de conexo  base de dados e abrir a conexo
                conn = GetConnection();

                // criar comando para insero
                OleDbCommand cmd = new OleDbCommand("INSERT INTO sales (customerid, dateofsale) VALUES (?, ?)", conn);
                cmd.Transaction = GetTransaction();
                cmd.Parameters.Add("custid", CustomerID);
                OleDbParameter parm = cmd.Parameters.Add("dt", OleDbType.Date);
                parm.Value = Date;

                // inserir registo 
                cmd.ExecuteNonQuery();

                // obter id autogerado
                OleDbCommand cmd2 = new OleDbCommand("SELECT @@IDENTITY", conn);
                cmd2.Transaction = GetTransaction();
                int id = (int)cmd2.ExecuteScalar();
                return id;
            }
            catch (OleDbException ex)
            {
                // tratar a excepo!!!!
                System.Console.WriteLine("EXCEPO no mtodo 'Insert': " + ex.Message);
                //status = ShopStatus.DatabaseError;
                throw new ShopException(ex);
            }
            finally
            {
                // fechar a conexo
                CloseConnection(conn);
            }
        }

        public void FillByCustomer(int customerID, DataSet ds)
        {
            OleDbConnection conn = null;

            try
            {
                // criar objecto de conexo  base de dados e abrir a conexo
                conn = GetConnection();

                // criar um Datadapter para executar o comando e devolver o dataset
                // cabealhos das vendas
                string sSqlCmd = "Select * From vwTotalBySale Where CustomerID = ?";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Transaction = GetTransaction();
                oAdapter.SelectCommand.Parameters.Add("custid", customerID);

                // executar o comando e preencher um Dataset
                oAdapter.Fill(ds, "Sales");
            }
            catch (OleDbException ex)
            {
                // tratar a excepo!!!!
                System.Console.WriteLine("EXCEPO no mtodo 'GetByCustomer': " + ex.Message);
                //status = ShopStatus.DatabaseError;
                throw new ShopException(ex);
            }
            finally
            {
                // fechar a conexo
                CloseConnection(conn);
            }
        }

        public void FillByID(int id, DataSet ds)
        {
            OleDbConnection conn = null;

            try
            {
                // criar objecto de conexo  base de dados e abrir a conexo
                conn = GetConnection();

                // criar um Datadapter para executar o comando e devolver o dataset
                // cabealhos das vendas
                string sSqlCmd = "SELECT * FROM vwTotalBySale WHERE SaleId=?";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Parameters.Add("sid", id);
                oAdapter.SelectCommand.Transaction = GetTransaction();

                // executar o comando e preencher um Dataset
                oAdapter.Fill(ds, "Sales");
            }
            catch (OleDbException ex)
            {
                // tratar a excepo!!!!
                System.Console.WriteLine("EXCEPO no mtodo 'GetSaleById': " + ex.Message);
                //status = ShopStatus.DatabaseError;
                throw new ShopException(ex);
            }
            finally
            {
                // fechar a conexo
                CloseConnection(conn);
            }
        }

        public void FillAll(DataSet ds)
        {
            OleDbConnection conn = null;

            try
            {
                // criar objecto de conexo  base de dados e abrir a conexo
                conn = GetConnection();

                // criar um Datadapter para executar o comando e devolver o dataset
                // cabealhos das vendas
                string sSqlCmd = "Select * From vwTotalBySale";
                OleDbDataAdapter oAdapter = new OleDbDataAdapter(sSqlCmd, conn);
                oAdapter.SelectCommand.Transaction = GetTransaction();

                // executar o comando e preencher um Dataset
                oAdapter.Fill(ds, "Sales");
            }
            catch (OleDbException ex)
            {
                // tratar a excepo!!!!
                System.Console.WriteLine("EXCEPO no mtodo 'GetAll': " + ex.Message);
                //status = ShopStatus.DatabaseError;
                throw new ShopException(ex);
            }
            finally
            {
                // fechar a conexo
                CloseConnection(conn);
            }
        }
    }
}
