using System;
using System.Data.OleDb;
using Isep.Util.Data;

namespace Shop.DAL
{
    // nao suporta nested transactions
    // ao efectuar commit ou rollback termina todo o contexto transacional
    public class TransactionalContext : IDisposable
    {
        public TransactionalContext() : this(false)
        {
        }

        public TransactionalContext(bool begin)
        {
            if (begin)
                Begin();
        }

        public void Begin()
        {
            if (!IsStarted)
            {
                theTransaction = OleDbHelper.CreateTransactedConnection(BaseGateway.CONN);
                theConnection = theTransaction.Connection;
            }
        }
        
        public void Commit()
        {
            if (!IsStarted)
                throw new InvalidOperationException("Transational context not defined");

            theTransaction.Commit();
            Cleanup();
        }

        public void Rollback()
        {
            if (!IsStarted)
                throw new InvalidOperationException("Transational context not defined");

            theTransaction.Rollback();
            Cleanup();
        }

        internal OleDbTransaction Transaction
        {
            get { return theTransaction; }
        }

        private bool IsStarted
        {
            get { return theTransaction != null; }
        }

        private OleDbConnection theConnection;
        private OleDbTransaction theTransaction;

        #region IDisposable Members

        public void Dispose()
        {
            if (IsStarted)
                Cleanup();
        }

        private void Cleanup()
        {
            //theTransaction.Rollback();
            theConnection.Close();
            theTransaction = null;
            theConnection = null;
        }

        #endregion
    }
}
